#include <pic.h>

	__CONFIG(INTIO & WDTEN & UNPROTECT & BOREN & MCLRDIS)

#define LCD

#include "types.h"

//--- PORT DEFINITIONS ---------------------------------------------------
//               76543210
#define  TrisA 0b00001000                
//               ||||||||
//               |||||||FR_LED
//               ||||||OV_LED
//               |||||BUF_OV_LED
//               ||||MODE
//               |||A0 
//               ||E
//               |---
//               ---

#define  TrisC 0b00000000
//               76543210
//               ||||||||
//               |||||||D4
//               ||||||D5
//               |||||D6
//               ||||D7
//               |||TX
//               ||RX
//               |---
//               ---

#define TX        RC4   //        - PIC-6
#define RX        RC5   //        - PIC-5
#define OV_LED    RA1   //        - PIC-12 
#define BUF_OV_LED RA2   //        - PIC-11
#define MODE      RA3   //        - PIC-4
#define FR_LED    RA0   //        - PIC-13 

#define LCD_A0   RA4   // LCD-4  - PIC-3
#define LCD_E    RA5   // LCD-6  - PIC-2
#define LCD_D4   RC0   // LCD-11 - PIC-10
#define LCD_D5   RC1   // LCD-12 - PIC-9
#define LCD_D6   RC2   // LCD-13 - PIC-8
#define LCD_D7   RC3   // LCD-14 - PIC-7

#define MODE_RAW 1
#define MODE_DATA 0

#include "delay.c"

#include "lcd.c"

#include "buffer.c"

#include "NMEA.c"

//------------------------------------------------------------------------------------

static volatile byte NMEA_line = 0;

static void interrupt
isr(void)			// Here be interrupt function - the name is unimportant.
{
  // handle the interrupt (insert application code here)

// needs 3-4 cycles to get into interrupt;

   if (RCIF) {
    char c;
    if (FERR) {
      FR_LED = 1;
      c = RCREG;
      c = RCREG;
      return;
    } else {
      FR_LED = 0;
    }

    if (OERR) {
      CREN = 0;
      c = RCREG;
      c = RCREG;
      CREN = 1;
      OV_LED = 1;
      return;
    } else {
      OV_LED = 0;
    }

    BUF_OV_LED = 0;

    c = RCREG;

    if (NMEA_line == 0) {
       if (c == '$') {
         NMEA_line=1;
       }
    } else {
       if (c == '*') {
         NMEA_line = 0;
       }
       BUF_OV_LED = putBuf (c) ? 0 : 1;
    }
  }

}

static bank1 byte PrintFirst = 1;

void main(void) {
  TRISA = TrisA;
  TRISC = TrisC;

  OPTION = 0b01000010; // pull-up, int/RA2 on 0->1; Prescalder 8 to TMR0;

  WPUA1 = 1; WPUA2 = 1;

  //         76453210
  OSCCON = 0b01110001; // 8 MHz clock;

  PORTA = 0; 
  PORTC = 0;

  CMCON0 = 0b00000111; // comparator is turned off;

  ANSEL = 0;

  ADCON1 = 0b01010000;      //  Select the Clock as Fosc/16

  ADIE = 0;

  ADCON0 = 0b10000000;     //  Turn off the ADC
                              //   Bit 7 - Right Justified Sample
                              //   Bit 6 - Dont Use VRef
                              //   Bit 4:2 - Channel 0
                              //   Bit 1 - Do not Start
                              //   Bit 0 - Turn on ADC  

  WDTCON = 0b10001; // 1000 = 1:8192 (1:16 relative to default)

  LCD_Init();          
  lcdWriteln ("Test");

  FR_LED = 1;
  Delay10ms (20);
  OV_LED = 1;
  Delay10ms (20);
  BUF_OV_LED = 1;
  Delay10ms (20);
  BUF_OV_LED = 0;
  Delay10ms (20);
  OV_LED = 0;
  Delay10ms (20);
  FR_LED = 0;
  Delay10ms (20);

  CLRWDT();

  SYNC = 0; // Asynch TX/RX;
//  BRGH = 1; BRG16= 1; SPBRG= 103; // 9600 baud rate;
//  BRGH = 1; BRG16= 0; SPBRG= 25; // 9600 baud rate (4 MHz);
  BRGH = 1; BRG16= 0; SPBRG= 51; // 9600 baud rate (8 MHz);

  SPEN = 1; // TX/RX Enabled;
  TXEN = 0; // TX;

  TXIE = 0;
  RCIE = 1;

  RX9  = 0; // Bit=8;

  CREN = 1; // RX;

  PEIE = 1; // peripheral interrupts enabled 
  GIE = 1; // interrupts allowed 

  for (;;) {
    char c = 0;

    while (!checkBuf()) {
      CLRWDT();
    }

    if (MODE == MODE_RAW) {

      byte iLine;

      byte iPos;

      lcdSetClearLine (1);
      lcdSetClearLine (0);

      iPos = 0;
      iLine = 0;
      for (;;) {
        CLRWDT();
        c = getBuf();
        if ((c == 0)) {
          Delay10ms(35);
          break;
        }

        if (iPos == 16) {
          iLine = ! iLine;
          if (iLine == 0) {
            Delay10ms (35);
            lcdSetClearLine (0);
          } else {
            lcdSetClearLine (1);
          }
          iPos = 0;
        } else 
          iPos++;

        lcdWrite (c);
      }

    } else {

//-- MODE != MODE_RAW -----------

      Error = 0;

      skipChar ('G');
      skipChar ('P');
      if (Error) goto HanleError;

      c = getChar();
      if (Error) goto HanleError;

      if (c == 'G') { 
//--- GPGGA ----------

        Lat_valid = 0;
        Long_valid = 0;
        Alt_valid = 0;
        Speed_valid = 0;
        Heading_valid = 0;

        skipChar ('G');
        skipChar ('A');
        skipChar (',');
        skipDomain (); // hhmmss.dd,
        if (Error) goto HanleError;

        Lat_valid = (getPosition(LAT) == 1); // xxmm.dddd , N or S
        Long_valid = (getPosition(LONG) == 1); // yyymm.dddd, E or W
        if (Error) goto HanleError1;

        if (getChar() == '0') // Fix Valid
          goto HanleError1;
        skipChar (',');

	SputnikN = getInt16(); // ss;
	Sputnik_valid = Int16_Valid;
//        skipDomain (); // ss

        skipDomain (); // d.d
        
	Alt = getInt16(); // ddd.d;
	Alt_valid = Int16_Valid;
        if (Error) goto HanleError1;

HanleError1:        
        if (PrintFirst) {
          lcdSetClearLine (0);
          lcdWriteln ("La ");
          if (Lat_valid) {
            WritePos (LAT);
          } else
            lcdWriteln ("???");

          lcdSetClearLine (1);
          lcdWriteln ("Lo ");
          if (Long_valid) 
            WritePos (LONG);
          else
            lcdWriteln ("???");

          Delay10ms (40);
       }

        skipTail();

//--- GPGGA ----------

      } else 

      if (c == 'V') { 

//--- GPVTG ----------

	Heading_valid = 0;
	Speed_valid = 0;

        skipChar ('T');
        skipChar ('G');
        skipChar (',');
        if (Error) goto HanleError2;

	Heading = getInt16(); // ddd.d;
	Heading_valid = Int16_Valid;
        if (Error) goto HanleError2;

        skipDomain (); // T,
        skipDomain (); // m,
        skipDomain (); // M,
        skipDomain (); // s.s,
        skipDomain (); // N,
        if (Error) goto HanleError2;

	Speed = getInt16(); // ddd.d;
	Speed_valid = Int16_Valid;

        skipDomain (); // K,
        if (Error) goto HanleError2;

        if (getChar() == 'A') // Data Valid
          goto HanleError2;

HanleError2:
        if (!PrintFirst) {
          lcdSetClearLine (0);
          lcdWrite ('A');
          if (Alt_valid) 
            WriteInt16 (Alt);
          else
            lcdWrite ('?');

          LCD_CURSOR_LINE_1(6)
          lcdWrite ('D');
          if (calcDist()) {
            WriteInt16 (Dist);
            LCD_CURSOR_LINE_1(12)
            lcdWrite ('B');
            WriteInt16 (BaseHeading);
          } else
            lcdWrite ('?');

          lcdSetClearLine (1);
          lcdWrite ('S');
          if (Speed_valid) 
            WriteInt16 (Speed);
          else
            lcdWrite ('?');

          LCD_CURSOR_LINE_2(6)
          lcdWrite ('V');
          if (Sputnik_valid) 
            WriteInt16 (SputnikN);
          else
            lcdWrite ('?');

          LCD_CURSOR_LINE_2(12)
          lcdWrite ('H');
          if (Heading_valid) 
            WriteInt16 (Heading);
          else
            lcdWrite ('?');
          Delay10ms (40);
        }
        PrintFirst = !PrintFirst;

        skipTail();

//--- GPVTG ----------

      } else {

          lcdSetClearLine (0);
          lcdWriteln("Else "); lcdWrite (c);
          Delay10ms (20);

      }

HanleError:

      continue;

//-- MODE != MODE_RAW -----------

    }

  } // for (;;)

}

