#define TEST

#include <math.h>

#define ERROR_EOL 1
#define ERROR_SYNTAX 2

static bank1 byte Error = 0;

#ifdef DEBUG_NMEA
static byte debug_pos = 0;

const char debug_NMEA[] = 
"GPGGA,200307.456,5955.4627,N,03022.9643,E,1,04,2.8,159.8,M,18.0,M,0.0,0000\0"
"GPVTG,201.3,T,180.9,M,0.4,N,3.7,K";

// GPGGA,200307.456,5955.4627,N,03022.9643,E,1,04,2.8,159.8,M,18.0,M,0.0,0000
// Time 200307.456
// Lat 5955.4627
// N
// Lng 03022.9643
// E
// Fix 1
// Sat 04
// H Dul 2.8
// Alt 159.8
// M
// Geoid Sep 18.0
// M
// DGPS AGE 0.0
// DGPS st id 0000

// GPVTG,201.3,T,180.9,M,0.4,N,0.7,K
// course 201.3
// T
// course 180.9
// M
// Speed 0.4
// N
// Speed 0.7
// K

#endif

char getChar(void) {
  char c;
  if (Error)
    return 0;

#ifndef DEBUG_NMEA

  c = getBuf();

#else
  c = debug_NMEA[debug_pos];

  debug_pos++;

  if (debug_pos == sizeof(debug_NMEA))
    debug_pos = 0;
#endif
  
  if ((c == 0) || (c == '*')) {
      Error = ERROR_EOL;
      return 0;
  }
//  lcdWrite (c);
  return c;
}

void skipChar(char target) {
  if (Error)
    return;
  if (getChar() != target) {
    Error = ERROR_SYNTAX;
  };
}

void skipTail(void) {
  if (Error == ERROR_EOL)
    return;
  Error = 0;
  while (getChar());
}

void skipTailDebug(void) {
  char c;
  byte i = 0;
  if (Error == ERROR_EOL)
    return;
  Error = 0;
  lcdSetClearLine (0);
  lcdWriteln ("S:");
  c = getChar();
  while (!Error) {
    if (i == 14) {
       lcdSetClearLine (1);
    }
    lcdWrite (c);
    c = getChar();
    i++;
  }
  Delay10ms (30);
}

void skipDomain(void) {
  if (Error == ERROR_EOL)
    return;
  for (;;) {
    char c = getChar();  
    if (Error)
      return;
    if (c == ',') {
      return;
    }
  }
}

#define LAT 0
#define LONG 1

typedef struct {
  int32 Pos;
  int16 Degree;
  uint8 Minute;
  uint16 FracMinute;
} Position;

static bank1 Position LatLong[2];

static bank1 int16 Alt;
static bank1 int16 Speed;
static bank1 int16 Heading;

static bank1 bit Lat_valid = 0;
static bank1 bit Long_valid = 0;
static bank1 bit Alt_valid = 0;
static bank1 bit Speed_valid = 0;
static bank1 bit Heading_valid = 0;

static bank1 int8 SputnikN;
static bank1 bit Sputnik_valid = 0;

byte getPosition(byte LAT_LONG) {
  char c;
  Position * p = &(LatLong[LAT_LONG]);

  c = getChar();
  if ((c == ',') || Error) {
    skipDomain();
    return 0;
  }

  if (LAT_LONG == LONG) {
    p->Degree = (c-'0')*100;
    c = getChar();
  } else {
    p->Degree = 0;
  }
  p->Degree += (c-'0')*10;
  p->Degree += (getChar()-'0');

  p->Minute = (getChar()-'0')*10; // minute
  p->Minute += (getChar()-'0');

  skipChar('.');

  p->FracMinute = ((uint16)(getChar()-'0'))*100;
  p->FracMinute += (getChar()-'0')*10;
  p->FracMinute += (getChar()-'0');
  if (getChar() >= '5') {
    if (p->FracMinute < 999) {
      p->FracMinute++;
    }
  }
  
  p->Pos = ( ((int32) p->Degree) * 60 + p->Minute ) * 1000 + p->FracMinute;  // (dg*60+minute)*1000+frac

  skipChar(',');
  if (getChar() == 'N') {
    p->Pos = -p->Pos;
    p->Degree = -p->Degree;
  }
  skipChar(',');

  return Error;
}

static bank1 bit Int16_Valid = 0;

int16 getInt16(void) { // ddd.d;
  byte neg;
  byte i;
  char c;

  Int16_Valid = 0;

  c = getChar();
  if ((c == ',') || Error) {
    return;
  }
  
  if (c == '-') {
    neg = 1;
    c = getChar();
  } else {
    neg = 0;
  }
  i = 0;
  for (;;) {
    if (Error)
      return 0;
    if (c == ',')
      break;
    if (c == '.') {
      break;
    }
    i = i*10+(c-'0');
    c = getChar();
  }
  if (Error)
    return 0;
  if (c == '.') {
    c = getChar();
    if (Error)
      return 0;
    if (c >= '5')
      i++;  
    skipDomain();
  }
  if (neg) 
    i = -i;

  Int16_Valid = 1;

  return i;
}

static bank1 bit baseSet = 0;

static bank1 int32 BaseLat;
static bank1 int32 BaseLong;
static bank1 double cosLat;
static bank1 int16 BaseHeading;

static bank1 int16 Dist;

#define	PI	3.14159265358979

byte calcDist (void) {
  double x;
  double y;
  double dx;
  double dy;

#ifndef TEST
  if (!Lat_valid || !Long_valid)
    return 0;
#endif

  if (!baseSet) {
#ifndef TEST
    BaseLat = LatLong[LAT].Pos;
    BaseLong = LatLong[LONG].Pos;
#else
    BaseLat = -3344648;
    BaseLong = -2278945;
#endif
    baseSet = 1;
//    cosLat = cos(((double)LatLong[LAT].Degree)+((double)LatLong[LAT].Minute)/60.0)*(PI/180);
    cosLat = 0.563;
  }
#ifdef TEST
  LatLong[LAT].Pos = -3345057;
  LatLong[LONG].Pos = -2279295;
#endif

  dx = LatLong[LONG].Pos-BaseLong;
  dy = LatLong[LAT].Pos-BaseLat;

  x = 11120.0/6000*cosLat*(dx);
  y = 11120.0/6000*(dy);
  Dist = (int16)(ceil(sqrt(x*x+y*y)));

  BaseHeading = -90-(int16)(ceil(180/PI*atan2 (dy, dx)));
  if (BaseHeading < 0)
    BaseHeading += 360;
  else
  if (BaseHeading > 360)
    BaseHeading -= 360;

  return 1;
}

void WritePos (byte LAT_LONG) {
  Position * p = &(LatLong[LAT_LONG]);
  WriteInt16 (p->Degree);
  lcdWrite ('\'');
  WriteInt16 (p->Minute);
  lcdWrite ('.');

 {
  uint16 n = p->FracMinute;
  char i;
  i = '0';
  while (n >= 100) {
    n -= 100;
    i++;
  }
  lcdWrite (i);

  i = '0';
  while (n >= 10) {
    n -= 10;
    i++;
  }
  lcdWrite (i);
  lcdWrite (n+'0');
 } 
}
 
const char fill [] = 
"GPGGA,200307.456,5955.4627,N,03022.9643,E,1,04,2.8,159.8,M,18.0,M,0.0,0000\0"
"GPGGA,200307.456,5955.4627,N,03022.9643,E,1,04,2.8,159.8,M,18.0,M,0.0,0000\0"
"GPGGA,200307.456,5955.4627,N,03022.9643,E,1,04,2.8,159.8,M,18.0,M,0.0,0000\0"
"GPGGA,200307.456,5955.4627,N,03022.9643,E,1,04,2.8,159.8,M,18.0,M,0.0,0000\0"
//"GPGGA,200307.456,5955.4627,N,03022.9643,E,1,04,2.8,159.8,M,18.0,M,0.0,0000\0"
//"GPGGA,200307.456,5955.4627,N,03022.9643,E,1,04,2.8,159.8,M,18.0,M,0.0,0000\0"
//"GPGGA,200307.456,5955.4627,N,03022.9643,E,1,04,2.8,159.8,M,18.0,M,0.0,0000\0"
//"GPGGA,200307.456,5955.4627,N,03022.9643,E,1,04,2.8,159.8,M,18.0,M,0.0,0000\0"
"GPGGA,200307.1211111111111111111111"
;

