#ifdef LCD

// LCD connector;
// 1   2   3   4   5   6   7   8   9   
// GND U+  A0  E   D4  D5  D6  D7  UBL+

//--- LCD FUNCTIONS ------------------------------------------------------

#define LCD_A0_mask 0b00100000
#define LCD_DD_mask 0b00001111

#define LCD_DELAY_BASE_100_US 1

void lcdSend4 (char c) {
  LCD_D4 = c & 0b0001 ? 1 : 0;
  LCD_D5 = c & 0b0010 ? 1 : 0;
  LCD_D6 = c & 0b0100 ? 1 : 0;
  LCD_D7 = c & 0b1000 ? 1 : 0;
//  PORTC = (c & LCD_DD_mask) | LCD_A0_mask; 
  LCD_E = 1; NOP(); LCD_E = 0;
  Delay100us (LCD_DELAY_BASE_100_US);
}

void lcdSend(char c) {
  // high data byte;
  LCD_D4 = c & 0b00010000 ? 1 : 0;
  LCD_D5 = c & 0b00100000 ? 1 : 0;
  LCD_D6 = c & 0b01000000 ? 1 : 0;
  LCD_D7 = c & 0b10000000 ? 1 : 0;
//  PORTC = (c1 & LCD_DD_mask) | LCD_A0_mask; 
  LCD_E = 1; NOP(); LCD_E = 0;
  Delay100us (LCD_DELAY_BASE_100_US);
  // low data byte;
  lcdSend4 (c);
}

void lcdWrite (char c) {
  LCD_A0 = 1;
  lcdSend (c);
}

void lcdCommand (char c) {
  LCD_A0 = 0;
  lcdSend (c);
}

void lcdCommand8 (char c) {
  lcdSend4 (c);
//  PORTC = c; LCD_E = 1; NOP(); LCD_E = 0;
//  Delay100us (LCD_DELAY_BASE_100_US);
}

#define LCD_CMD_POS1		( 0x80 )
#define LCD_CMD_POS2		(0x80 | 0x40)

#define LCD_CURSOR_ON		lcdCommand (0xC | 0x2);
#define LCD_CURSOR_OFF		lcdCommand (0xC);
#define LCD_CURSOR_BLINK_ON	lcdCommand (0xC | 0x2 | 0x1);
#define LCD_CLEAR		{ lcdCommand (0x1); Delay100us (15+LCD_DELAY_BASE_100_US); }
#define LCD_CURSOR_HOME		lcdCommand (0x2);
#define LCD_SHIFT_CURSOR_LEFT	lcdCommand (0x10);
#define LCD_SHIFT_CURSOR_RIGHT	lcdCommand (0x10 | 0x4);
#define LCD_SHIFT_DISPLAY_LEFT	lcdCommand (0x10 | 0x8);
#define LCD_SHIFT_DISPLAY_RIGHT	lcdCommand (0x10 | 0x8 | 0x4);
#define LCD_CURSOR_LINE_1(x)	lcdCommand (LCD_CMD_POS1 | x);
#define LCD_CURSOR_LINE_2(x)	lcdCommand (LCD_CMD_POS2 | x);

void lcdProgram (const char * p) {
  byte i;
  char c;
  char lbCommand = 1;
  for (i = 0; ; i++) {
    char c = p[i];
    if (c == 0)
      return;
    if (lbCommand) {
      lcdCommand (c);
      if (c & LCD_CMD_POS1) // output character after cursor placement command;
        lbCommand = 0;
    } else {
      lcdWrite (c);
      lbCommand = 1;
    }
  }
}

void lcdWriteln (const char * p) {
  byte i;
  for (i = 0; p[i] != 0; i++) {
      lcdWrite(p[i]);
  }
}

const char LCD_Data_Init[] = {
  0x28, //  
  0x0C, //  
  0x6,  //     
  0x0F, //     
//  0x80, //  
//  ' ',
  0x01,  // ,  
  0
};

void LCD_Init(void) { // 16s2d
  LCD_E = 0;
  LCD_A0 = 0;
  Delay10ms (5);

// 30h  
  lcdCommand8 (3);
  lcdCommand8 (3);
  lcdCommand8 (3);

// 20h  
  lcdCommand8 (2);

//4bit
  lcdProgram (LCD_Data_Init);
  Delay100us (15);
}

void WriteNum (uint16 n) {
  byte i;
  i = '0';
  while (n >= 1000) {
    n -= 1000;
    i++;
  }
  lcdWrite (i);

  i = '0';
  while (n >= 100) {
    n -= 100;
    i++;
  }
  lcdWrite (i);

  i = '0';
  while (n >= 10) {
    n -= 10;
    i++;
  }
  lcdWrite (i);
  lcdWrite (n+'0');
}

void WriteBNum (byte n) {
  byte i;
  i = '0';
  while (n >= 100) {
    n -= 100;
    i++;
  }
  lcdWrite (i);

  i = '0';
  while (n >= 10) {
    n -= 10;
    i++;
  }
  lcdWrite (i);
  lcdWrite (n+'0');
}

void lcdSetClearLine (byte iLine) {
  char i;
  byte lcdCmd = (iLine == 0) ? LCD_CMD_POS1 : LCD_CMD_POS2;
  lcdCommand (lcdCmd);
  for (i = 0; i < 16; i++) 
    lcdWrite (' ');
  lcdCommand (lcdCmd);
}

static bit first_char;

void Write_D (byte i) {
  if (first_char || (i)) {
    lcdWrite (i+'0');
    first_char = 1;
  }
}

void _WriteUInt16 (uint16 n) {
  byte i;
  i = 0;
  while (n >= 10000) {
    n -= 10000;
    i++;
  }
  Write_D (i);

  i = 0;
  while (n >= 1000) {
    n -= 1000;
    i++;
  }
  Write_D (i);

  i = 0;
  while (n >= 100) {
    n -= 100;
    i++;
  }
  Write_D (i);

  i = 0;
  while (n >= 10) {
    n -= 10;
    i++;
  }
  Write_D (i);

  first_char = 1;
  Write_D (n);
}


void WriteInt16 (int16 n) {
  byte i;
  first_char = 0;
  if (n < 0) {
    lcdWrite ('-');
    n = -n;
  }
  _WriteUInt16 (n);
}

/*
void WriteInt32 (int32 n) {
  byte i;
  if (n < 0) {
    lcdWrite ('-');
    n = -n;
  }

// 2.147.483.647

  first_char = 0;

  i = 0;    //9876543210
  while (n >= 1000000000L) {
    n -=  1000000000L;
    i++;//9876543210
  }
  Write_D (i);

  i = 0;    //876543210
  while (n >= 100000000L) {
    n -=  100000000L;
    i++;//876543210
  }
  Write_D (i);

  i = 0;    //76543210
  while (n >= 10000000L) {
    n -=  10000000L;
    i++;//76543210
  }
  Write_D (i);

  i = 0;    //6543210
  while (n >= 1000000L) {
    n -=  1000000L;
    i++;//6543210
  }
  Write_D (i);

  i = 0;    //543210
  while (n >= 100000L) {
    n -=  100000L;
    i++;//543210
  }
  Write_D (i);

  i = 0;    //43210
  while (n >= 10000L) {
    n -=  10000L;
    i++;//54210
  }
  Write_D (i);

  i = 0;
  while (n >= 1000) {
    n -= 1000;
    i++;
  }
  Write_D (i);

  i = 0;
  while (n >= 100) {
    n -= 100;
    i++;
  }
  Write_D (i);

  i = 0;
  while (n >= 10) {
    n -= 10;
    i++;
  }
  Write_D (i);

  first_char = 1;

  Write_D (n);
}
*/

#endif

